package cn.com.qmth.stmms.ms.admin.api;

import cn.com.qmth.stmms.ms.core.domain.CheckData;
import cn.com.qmth.stmms.ms.core.domain.enums.CheckDataStatusEnum;
import cn.com.qmth.stmms.ms.core.domain.enums.CheckItemEnum;
import cn.com.qmth.stmms.ms.core.repository.CheckDataRepo;
import cn.com.qmth.stmms.ms.marking.service.CheckDataService;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * 数据校验
 */
@RestController
@RequestMapping("/api/check_data")
public class CheckDataApi {

    @Resource
    CheckDataService checkDataService;

    @Resource
    CheckDataRepo checkDataRepo;

    /**
     * 获取同步结果
     *
     * @param workId 工作ID
     */
    @GetMapping("/list")
    public List<CheckData> list(@RequestParam Long workId) {
        return checkDataRepo.findByWorkId(workId);
    }

    /**
     * 检查
     *
     * @param workId     工作ID
     * @param checkItem  数据类型
     * @param paramValue 变量参数
     */
    @RequestMapping(value = "/check", method = RequestMethod.POST)
    public Object check(@RequestParam Long workId,
                        @RequestParam CheckItemEnum checkItem,
                        @RequestParam(required = false) Integer paramValue) {
        CheckData checkData = checkDataRepo.findByWorkIdAndCheckItem(workId, checkItem);
        if (checkData == null) {
            checkData = new CheckData(workId, checkItem, paramValue);
        } else {
            if (!CheckDataStatusEnum.FINISH.equals(checkData.getStatus())) {
                throw new RuntimeException("有任务正在执行");
            }
            checkData.resetData(paramValue);
        }
        checkDataRepo.saveAndFlush(checkData);

        // 异步任务开始
        checkDataService.syncCheckData(checkData);

        return true;
    }
}
