package cn.com.qmth.stmms.ms.admin.service;

import cn.com.qmth.stmms.ms.commons.utils.DbBackupUtils;
import cn.com.qmth.stmms.ms.core.cache.CacheService;
import cn.com.qmth.stmms.ms.core.cache.ParamCache;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.Work;
import cn.com.qmth.stmms.ms.core.domain.enums.TrialEnum;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import cn.com.qmth.stmms.ms.marking.service.MarkingService;
import cn.com.qmth.stmms.ms.marking.service.StageControlService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 试评任务
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2019/11/4
 */
@Service
public class TrialService {

    @Autowired
    WorkRepo workRepo;

    @Autowired
    MarkSubjectRepo markSubjectRepo;

    @Autowired
    StudentRepo studentRepo;

    @Resource
    MarkTaskLevelRepo markTaskLevelRepo;

    @Resource
    MarkTaskRoughLevelRepo markTaskRoughLevelRepo;

    @Autowired
    PaperRepo paperRepo;

    @Autowired
    StageControlService stageControlService;

    @Autowired
    ExamQuestionRepo examQuestionRepo;

//    @Autowired
//    MarkLogRepo markLogRepo;

    @Autowired
    MarkerGroupRepo markerGroupRepo;

    @Autowired
    MarkingService markingService;

    @Autowired
    DbBackupUtils dbBackupUtils;

    @Autowired
    CacheService cacheService;

    @Autowired
    MarkUserRepo markUserRepo;

    @Autowired
    MarkerGroupLeaderRepo markerGroupLeaderRepo;

    @Resource
    MarkerGroupStudentRepo markerGroupStudentRepo;

    /**
     * 检查任务状态
     *
     * @param workId  工作ID
     * @param subject 科目
     */
    @Transactional
    public void checkMissionStatus(Long workId, Subject subject) {
        Work work = workRepo.findOne(workId);
        if (Objects.isNull(work)) {
            throw new RuntimeException("没有此工作区");
        }
        MarkSubject markSubject = markSubjectRepo.findOne(workId + "-" + subject.name());
        if (markSubject.getTest() == TrialEnum.INIT.getId() || markSubject.getTest() == TrialEnum.START_TRIAL.getId()) {
            finishTrial(workId, subject);
        }
    }

    /**
     * 开始试评任务
     *
     * @param workId  工作Id
     * @param subject 科目
     */
    @Transactional
    public void startTrial(Long workId, Subject subject) throws Exception {
        Work work = workRepo.findOne(workId);
        if (Objects.isNull(work)) {
            throw new RuntimeException("没有此工作区");
        }

        List<MarkSubject> markSubjectList = work.getSubjects().stream().filter(o -> o.getId().toUpperCase().contains(subject.toString().toUpperCase())).collect(Collectors.toList());
        MarkSubject markSubject = markSubjectList.get(0);
        //分配任务
        Map<String, Integer> map = new HashMap();
        map.put("taskCount", 0);
        stageControlService.createTask(markSubject, map);
        //修改科目test
        markSubject.setTest(TrialEnum.START_TRIAL.getId());
        markSubjectRepo.save(markSubject);

        //修改试卷test
        paperRepo.updateByWorkIdAndSubjectAndTest(workId, subject, TrialEnum.INIT.getId(), TrialEnum.START_TRIAL.getId());

        // 清除分组中导入的名单
        markerGroupStudentRepo.deleteByWorkIdAndSubjectAndStageAndUsed(workId, subject, markSubject.getStage(), false);
    }

    /**
     * 结束试评任务，保留标准卷
     *
     * @param workId  工作ID
     * @param subject 科目
     */
    @Transactional
    public void finishTrial(Long workId, Subject subject) {
        Work work = workRepo.findOne(workId);
        if (Objects.isNull(work)) {
            throw new RuntimeException("没有此工作区");
        }
        //更新科目为初始状态并设置test为false
        MarkSubject markSubject = markSubjectRepo.findOne(workId + "-" + subject.name());
        markSubject.setTest(TrialEnum.DEFAULT.getId());
        // 开启粗分档且当前在粗分档试评,则重置为INIT
        // 开启粗分档且当前在细分档试评,则重置为ROUGH_LEVEL
        // 未开启粗分档(当前只能在细分档试评),则重置为INIT
        boolean roughLevel = ParamCache.levelConfigMap.get(String.valueOf(workId)).getRoughLevelEnable() == 1;
        MarkStage stage = roughLevel && MarkStage.LEVEL.equals(markSubject.getStage()) ? MarkStage.ROUGH_LEVEL : MarkStage.INIT;
        markSubject.setStage(stage);
        markSubject.setSampleCount(0);
        markSubjectRepo.save(markSubject);
        //删除该科目下所有任务数据
        if (roughLevel && MarkStage.INIT.equals(markSubject.getStage())) {
            markTaskRoughLevelRepo.deleteByWorkIdAndSubject(workId, subject);
        } else {
            markTaskLevelRepo.deleteByWorkIdAndSubject(workId, subject);
        }


        //更新该科目下的试卷信息(正式卷子，并删除试评卷子)
        List<Paper> paperList = paperRepo.findByWorkIdAndSubjectAndTest(workId, subject, TrialEnum.START_TRIAL.getId());

        for (Paper paper : paperList) {
            // 标准卷不清除档位
            paper.setActive(false);
            paper.setBatchNo(null);
            // 区分粗分档和细分档
            if (roughLevel && MarkStage.INIT.equals(markSubject.getStage())) {
                paper.setRoughLevel(paper.isRoughSample() ? paper.getRoughLevel() : null);
            } else {
                paper.setLevel(paper.isSample() ? paper.getLevel() : null);
            }
            paper.setTest(TrialEnum.DEFAULT.getId());
        }

        //更新试评状态
        if (!CollectionUtils.isEmpty(paperList)) {
            List<Paper> data = new ArrayList<>();
            //1000条提交一次
            for (Paper paper : paperList) {
                if (data.size() == 1000) {
                    paperRepo.save(data);
                    data.clear();
                }
                data.add(paper);
            }
            //将剩下的数据也导入
            if (!data.isEmpty()) {
                paperRepo.save(data);
            }
        }

        // 删除分组,删除分组内成员
        String backupPrefix = "";
        if (roughLevel && MarkStage.INIT.equals(markSubject.getStage())) {
            backupPrefix = "粗分档";
            markerGroupRepo.deleteByWorkIdAndSubjectAndStage(workId, subject, MarkStage.ROUGH_LEVEL);
            markerGroupLeaderRepo.deleteByWorkIdAndSubjectAndStage(workId, subject, MarkStage.ROUGH_LEVEL);
        } else {
            backupPrefix = "细分档";
            markerGroupRepo.deleteByWorkIdAndSubjectAndStage(workId, subject, MarkStage.LEVEL);
            markerGroupLeaderRepo.deleteByWorkIdAndSubjectAndStage(workId, subject, MarkStage.LEVEL);
        }

        // 所有用户强制退出
        List<MarkUser> userList = markUserRepo.findByWorkIdAndSubject(workId, subject);
        userList = userList.stream().filter(m -> Role.MARKER.equals(m.getRole()) || Role.MARK_LEADER.equals(m.getRole())).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(userList)) {
            for (MarkUser user : userList) {
                cacheService.deleteTokenCache(user.getId().toString());
            }
        }
        dbBackupUtils.startBackup(workId, backupPrefix + "试评");
    }
}
