package cn.com.qmth.stmms.ms.marking.service;

import cn.com.qmth.stmms.ms.commons.lock.LockService;
import cn.com.qmth.stmms.ms.commons.lock.LockType;
import cn.com.qmth.stmms.ms.commons.utils.ServletUtil;
import cn.com.qmth.stmms.ms.commons.web.PageableDTO;
import cn.com.qmth.stmms.ms.core.cache.ParamCache;
import cn.com.qmth.stmms.ms.core.domain.ExamQuestion;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.enums.TrialEnum;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskRoughLevel;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import cn.com.qmth.stmms.ms.marking.assembler.MarkTaskRoughLevelAssembler;
import cn.com.qmth.stmms.ms.marking.assembler.PaperAssembler;
import cn.com.qmth.stmms.ms.marking.dto.MarkTaskDTO;
import cn.com.qmth.stmms.ms.marking.dto.PaperDTO;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.persistence.criteria.Join;
import javax.persistence.criteria.JoinType;
import javax.persistence.criteria.Predicate;
import java.util.*;

/**
 * 评卷工作服务
 */
@Service
public class MarkTaskRoughLevelService {

    @Resource
    private MarkUserRepo markUserRepo;

    @Resource
    private PaperRepo paperRepo;

    @Resource
    private MarkSubjectRepo markSubjectRepo;

    @Resource
    private MarkTaskRoughLevelRepo markTaskRoughLevelRepo;

    @Resource
    private ExamQuestionRepo examQuestionRepo;

    @Resource
    private MarkTaskRoughLevelAssembler markTaskRoughLevelAssembler;

    @Resource
    MarkingRoughLevelService markingRoughLevelService;

    @Resource
    private LockService lockService;

    @Resource
    private PaperAssembler paperAssembler;


    public PageableDTO listMarkTaskLevel(Long markerId, Boolean isSample, Boolean reject, String level, Long questionId, String areaCode, Pageable pageable) {
        Long workId = ServletUtil.getWordId();
        List<MarkTaskDTO> markTaskDTOs = new ArrayList<>();
        MarkUser markUser = markUserRepo.findOne(markerId);
        Long batchNo = null;
        List<Object> batchNos = paperRepo.findRoughBatchNoByWorkIdAndSubject(workId, markUser.getSubject().name());
        if (batchNos != null && !batchNos.isEmpty()) {
            Object object = batchNos.get(0);
            batchNo = Long.valueOf(object.toString());
        }

        Long finalBatchNo = batchNo;
        Integer levelShowAllPaper = ParamCache.levelConfigMap.get(String.valueOf(workId)).getLevelShowAllPaper();
        Specification<MarkTaskRoughLevel> specification = (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            List<Predicate> onPredicates = new ArrayList<>();
            Join<MarkTaskRoughLevel, Paper> join = root.join("paper", JoinType.INNER);
            if (Objects.nonNull(questionId)) {
                predicates.add(builder.equal(root.get("questionId"), questionId));
            }
            predicates.add(builder.equal(root.get("markerId"), markerId));
            if (level == null) {
                predicates.add(builder.isNull(root.get("result")));
            } else {
                //查询
                predicates.add(builder.equal(root.get("result"), level));
                if (levelShowAllPaper == 0) {
                    if (!Objects.isNull(finalBatchNo)) {
                        onPredicates.add(builder.equal(join.get("roughBatchNo"), finalBatchNo));
                    }
                }
            }
            if (isSample != null) {
                onPredicates.add(builder.equal(join.get("isRoughSample"), isSample));
            }
            if (reject != null && reject) {
                onPredicates.add(builder.equal(join.get("isRejected"), reject));
            } else {
                predicates.add(builder.equal(root.get("isRejected"), reject));
            }
            onPredicates.add(builder.isNotNull(join.get("roughBatchNo")));
            //过滤考区
            if (Objects.nonNull(areaCode)) {
                onPredicates.add(builder.equal(join.get("areaCode"), areaCode));
            }
            join.on(onPredicates.toArray(new Predicate[onPredicates.size()]));
            return builder.and(predicates.toArray(new Predicate[predicates.size()]));
        };
        Sort sort = new Sort("randomSeq", "randomSeqNew");
        Pageable pageable1 = new PageRequest(pageable.getPageNumber(), pageable.getPageSize(), sort);
        Page<MarkTaskRoughLevel> markTasks = markTaskRoughLevelRepo.findAll(specification, pageable1);

        if (Objects.isNull(markTasks) || markTasks.getContent().isEmpty()) {
            if (Objects.nonNull(isSample) && isSample) {
                ExamQuestion examQuestion = examQuestionRepo.findOne(questionId);
                MarkSubject markSubject = markSubjectRepo.findOne(workId + "-" + examQuestion.getSubject());
                if (markSubject.getTest() == TrialEnum.DEFAULT.getId()) {
                    List<Paper> paperList = paperRepo.findSample(level, isSample, false, examQuestion.getSubject().name(), TrialEnum.DEFAULT.getId(), examQuestion.getAreaCode(), examQuestion.getWorkId());
                    if (Objects.nonNull(paperList) && !paperList.isEmpty()) {
                        for (Paper p : paperList) {
                            MarkTaskRoughLevel markTask = new MarkTaskRoughLevel(markUser, p, MarkStage.LEVEL, 1L);
                            markTaskDTOs.add(markTaskRoughLevelAssembler.toDTO(markTask));
                        }
                    }
                }
            }
        }
        markTasks.getContent().forEach(m -> {
            markTaskDTOs.add(markTaskRoughLevelAssembler.toDTO(m));
        });
        if (Objects.isNull(level)) {
            randomSeqNewSort(markTaskDTOs);
        }
        return new PageableDTO(markTaskDTOs, markTasks.getTotalElements(), markTasks.getTotalPages(), pageable.getPageNumber());
    }


    public void randomSeqNewSort(List<MarkTaskDTO> markTaskDTOs) {
        boolean isSort = true;
        for (MarkTaskDTO m : markTaskDTOs) {
            if (Objects.nonNull(m.getRandomSeq())) {
                isSort = false;
                break;
            }
        }
        if (isSort) {
            markTaskDTOs.sort(Comparator.comparing(MarkTaskDTO::getRandomSeqNew));
        }
    }

    public ResponseEntity markingLevel(Long markTaskId, HashMap<String, String> body) {
        MarkTaskRoughLevel markTask = markTaskRoughLevelRepo.findOne(markTaskId);
        String result = body.get("result");
        boolean oldRejected = markTask.isRejected();
        boolean oldShift = markTask.getPaper().isShift();
        boolean oldShiftScore = markTask.getPaper().isShiftScore();
        if (result == null) {
            return new ResponseEntity(HttpStatus.BAD_REQUEST);
        }
        try {
            lockService.waitlock(LockType.ROUGH_LEVEL, markTask.getPaper().getId());
            markingRoughLevelService.levelMark(markTask, result);
        } catch (Exception e) {
            throw new RuntimeException("分档异常");
        } finally {
            lockService.unlock(LockType.ROUGH_LEVEL, markTask.getPaper().getId());
        }
        return new ResponseEntity(markTaskRoughLevelAssembler.toDTO(markTask, oldRejected, oldShift, oldShiftScore), HttpStatus.OK);
    }

    public ResponseEntity skipLevel(Long markTaskId) {
        MarkTaskRoughLevel markTask = markingRoughLevelService.skip(markTaskId);
        return new ResponseEntity(markTaskRoughLevelAssembler.toDTO(markTask), HttpStatus.OK);
    }

    public boolean updateMarkTaskMarkById(Long markTaskId, Boolean isMark) {
        markTaskRoughLevelRepo.updateMarkTaskMarkById(markTaskId, isMark);
        return true;
    }

    public List<MarkTaskDTO> reviewPaperLevel(Long markerId, Long questionId, String areaCode) {
        List<MarkTaskDTO> markTaskDTOs = new ArrayList<>();
        Sort sort = new Sort(Sort.Direction.DESC, "updatedOn");
        Pageable pageable = new PageRequest(0, 5, sort);
        Long batchNo = paperRepo.findByQuestionId(questionId);
        ;

        Long finalBatchNo = batchNo;
        Specification<MarkTaskRoughLevel> specification = (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            if (Objects.nonNull(questionId)) {
                predicates.add(builder.equal(root.get("questionId"), questionId));
            }
            predicates.add(builder.equal(root.get("markerId"), markerId));
            //查询
            if (!Objects.isNull(finalBatchNo)) {
                predicates.add(builder.equal(root.get("paper").get("batchNo"), finalBatchNo));
            }
            predicates.add(builder.isNotNull(root.get("result")));
            //分档需要过滤标准卷
            predicates.add(builder.equal(root.get("paper").get("isSample"), false));
            //过滤考区
            if (Objects.nonNull(areaCode)) {
                predicates.add(builder.equal(root.get("paper").get("areaCode"), areaCode));
            }
            return builder.and(predicates.toArray(new Predicate[predicates.size()]));
        };
        Page<MarkTaskRoughLevel> markTasks = markTaskRoughLevelRepo.findAll(specification, pageable);
        markTasks.getContent().forEach(m -> {
            markTaskDTOs.add(markTaskRoughLevelAssembler.toDTO(m));
        });
        return markTaskDTOs;
    }

    public PageableDTO listMarkedTaskLevel(Long workId, Subject subject, Long markerId, Pageable pageable) {
        MarkUser markUser = markUserRepo.findOne(markerId);
        if(Role.MARKER.equals(markUser.getRole())) {
            List<PaperDTO> paperDTOS = new ArrayList<>();
            Specification<MarkTaskRoughLevel> specification = (root, query, builder) -> {
                List<Predicate> predicates = new ArrayList<>();
                predicates.add(builder.equal(root.get("workId"), workId));
                predicates.add(builder.equal(root.get("subject"), subject));
                predicates.add(builder.equal(root.get("stage"), MarkStage.ROUGH_LEVEL));
                predicates.add(builder.equal(root.get("markerId"), markerId));
                predicates.add(builder.equal(root.get("isMark"), true));
                return builder.and(predicates.toArray(new Predicate[0]));
            };
            Page<MarkTaskRoughLevel> markTasks = markTaskRoughLevelRepo.findAll(specification, pageable);

            markTasks.getContent().forEach(m -> paperDTOS.add(paperAssembler.toDTO(m.getPaper())));
            return new PageableDTO(paperDTOS, markTasks.getTotalElements(), markTasks.getTotalPages(), pageable.getPageNumber());
        } else if(Role.MARK_LEADER.equals(markUser.getRole())){
            List<PaperDTO> paperDTOS = new ArrayList<>();
            Specification<Paper> specification = (root, query, builder) -> {
                List<Predicate> predicates = new ArrayList<>();
                predicates.add(builder.equal(root.get("workId"), workId));
                predicates.add(builder.equal(root.get("subject"), subject));
                predicates.add(builder.equal(root.get("isMark"), true));
                return builder.and(predicates.toArray(new Predicate[0]));
            };
            Page<Paper> papers = paperRepo.findAll(specification, pageable);

            papers.getContent().forEach(m -> paperDTOS.add(paperAssembler.toDTO(m)));
            return new PageableDTO(paperDTOS, papers.getTotalElements(), papers.getTotalPages(), pageable.getPageNumber());
        } else {
            throw new RuntimeException("当前角色无法查询标记试卷");
        }
    }

    public List<MarkTaskRoughLevel> getBySecretNumberLevel(Long workId, Subject subject, String secretNumber, int test) {
        return markTaskRoughLevelRepo.findByWorkIdAndSubjectAndSecretNumberAndTest(workId, subject, secretNumber, test);
    }

    public PageableDTO shiftLevel(Long markerId, Long workId, Boolean isShift, Boolean isShiftScore, Long questionId, Pageable pageable) {
        List<MarkTaskDTO> markTaskDTOs = new ArrayList<>();
        Specification<MarkTaskLevel> specification = (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(builder.equal(root.get("questionId"), questionId));
            predicates.add(builder.equal(root.get("markerId"), markerId));
            //查询
            predicates.add(builder.equal(root.get("paper").get("isShift"), isShift));
            predicates.add(builder.equal(root.get("stage"), MarkStage.LEVEL));
            predicates.add(builder.isNull(root.get("result")));
            return builder.and(predicates.toArray(new Predicate[predicates.size()]));
        };
        Sort sort = new Sort("paper.level", "serialNumber", "randomSeq");
        Pageable pageable1 = new PageRequest(pageable.getPageNumber(), pageable.getPageSize(), sort);
        Page<MarkTaskRoughLevel> markTasks = markTaskRoughLevelRepo.findAll(specification, pageable1);

        markTasks.getContent().forEach(m -> markTaskDTOs.add(markTaskRoughLevelAssembler.toShiftDTO(m)));
        return new PageableDTO(markTaskDTOs, markTasks.getTotalElements(), markTasks.getTotalPages(), pageable.getPageNumber());
    }

    public PaperDTO getByTaskSecretNumberLevel(Long questionId, String sn) {
        PaperDTO paperDTO = null;
        Specification<MarkTaskLevel> specification = (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(builder.equal(root.get("paper").get("questionId"), questionId));
            if (sn != null) {
                predicates.add(builder.equal(root.get("secretNumber"), sn));
            }
            return builder.and(predicates.toArray(new Predicate[predicates.size()]));
        };
        Optional<MarkTaskRoughLevel> taskOptional = markTaskRoughLevelRepo.findAll(specification, new PageRequest(0, 1)).getContent().stream().findFirst();
        if (taskOptional.isPresent()) {
            paperDTO = paperAssembler.toDTO(taskOptional.get().getPaper());
        }
        return paperDTO;
    }
}
