package cn.com.qmth.stmms.ms.marking.service;

import cn.com.qmth.stmms.ms.commons.config.LevelConfig;
import cn.com.qmth.stmms.ms.commons.constant.ArbitrateCallback;
import cn.com.qmth.stmms.ms.commons.constant.ArbitrateResult;
import cn.com.qmth.stmms.ms.core.cache.ParamCache;
import cn.com.qmth.stmms.ms.core.domain.*;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskRoughLevel;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.marking.service.arbitration.ArbitrationService;
import com.alibaba.fastjson.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 评卷，分档仲裁及打分服务
 * Created by zhengmin on 2016/9/23.
 */
@Service
public class MarkingRoughLevelService {

    private static final Logger LOG = LoggerFactory.getLogger(MarkingRoughLevelService.class);

    @Autowired
    private DetermineRoughLevelService determineRoughLevelService;

    @Resource
    private MarkTaskRoughLevelRepo markTaskRoughLevelRepo;

    @Resource
    private MarkTaskLevelRepo markTaskLevelRepo;

    @Autowired
    private MarkUserRepo markUserRepo;

    @Autowired
    private PaperRepo paperRepo;

    @Autowired
    private LevelRepo levelRepo;

    @Autowired
    MarkingRoughLevelService markingService;

    @Autowired
    MarkLogRepo markLogRepo;

    @Autowired
    WorkRepo workRepo;

    @Autowired
    LevelConfig levelConfig;

    @Autowired
    ArbitrateCallback arbitrateCallback;

    @Resource
    MarkTaskJobRepo markTaskJobRepo;

    @Autowired
    MarkerGroupRepo markerGroupRepo;

    @Autowired
    MarkerGroupLeaderRepo markerGroupLeaderRepo;

    @Autowired
    TaskPublishSettingRepo taskPublishSettingRepo;

    @Autowired
    ChangeLevelRepo changeLevelRepo;

    @Autowired
    AssignTaskService assignTaskService;

    /**
     * 提交分档。如果每个评卷都提交了，则进行仲裁判定，如果没有触发仲裁就定档
     *
     * @param markTask
     * @param levelCode
     * @return
     */
    @Transactional
    public MarkTaskRoughLevel levelMark(MarkTaskRoughLevel markTask, String levelCode) {
        long start = System.currentTimeMillis();
        LOG.info("提交分档:{}", start);

        //如果已经由科组长定档，则不做处理
        Paper paper = markTask.getPaper();
        if (paper.isMarkByLeader() && Objects.nonNull(paper.getRoughLevel())) {
            return markTask;
        }

        MarkUser maker = markUserRepo.findOne(markTask.getMarkerId());
        markTask.setResult(levelCode);
        markTask.setLevelValue(Integer.valueOf(levelCode));
        markTask.setRejected(false);
        markTask.setDeviationDirection(null);
        markTask.setUpdatedOn(new Date());
        markTask.setMarkerName(maker.getName());
        //激活试卷
//        markTask.setActive(true);

        markTaskRoughLevelRepo.save(markTask);

        //判断该任务的试卷是否所有评卷员都评完，要进行仲裁判定或定档
        List<MarkTaskRoughLevel> markTasks = markTaskRoughLevelRepo.findByPaperId(paper.getId());
        long leftCount = markTasks.stream().filter(i -> i.getResult() == null).count();
        if (leftCount == 0) {
            List<Level> levels = levelRepo.findByWorkId(paper.getWorkId());
            //定档
            RoughDeterResult determine = determineRoughLevelService.determine(markTask.getWorkId(), levels, markTasks);
            if (Objects.nonNull(determine) && Objects.nonNull(determine.getResult())) {
                markTask.setDateMineResult(determine.getResult());
            }
            //1.过半定档
            if (Objects.nonNull(determine) && Objects.nonNull(determine.getResult()) && determine.getDeterType() == DetermineRoughLevelService.DeterType.MAJORITY) {
                paper.determineRoughLevel(determine.getResult());
                paper.setMarkedLogic(false);
                //档位落差值日志
                levelsLog(markTasks, paper);
            } else {
                int[] values = markTasks.stream().mapToInt(MarkTaskRoughLevel::getLevelValue).toArray();
                Integer deviation = Optional.ofNullable(ParamCache.levelConfigMap.get(String.valueOf(markTask.getWorkId())).getDeviation()).orElse(3);
                boolean result = ArbitrationService.arbitrate(deviation, values);
                //2.最大落差超过仲裁档位,则仲裁打回
                if (result) {
                    //仲裁
                    paper.arbitrate();
                }
                //3.最大落差未超过仲裁档位,则权重计算定档
                else if (Objects.nonNull(determine) && Objects.nonNull(determine.getResult()) && determine.getDeterType() == DetermineRoughLevelService.DeterType.WEIGHT) {
                    paper.determineRoughLevel(determine.getResult());
                    paper.setMarkedLogic(true);
                    //档位落差值日志
                    levelsLog(markTasks, paper);
                }
            }
            //改档后重新分档，状态改为false
            paper.setShift(false);
            //重新分档，打回科组长打回状态改为false
            paper.setRejectedByLeader(false);
            paper.setSortNum(calcSortNum(paper, markTasks, levels));
            paperRepo.save(paper);

            //同步mark_task表中level
            markTaskRoughLevelRepo.updateLevelByPaperId(paper.getRoughLevel(), paper.getId());
        }
        long end = System.currentTimeMillis();
        LOG.info("提交分档耗时：{}", (end - start) / 1000 + "s");
        return markTask;
    }

    /**
     * 档位落差值计算日志
     *
     * @param markTasks
     * @param paper
     */
    @Transactional
    public void levelsLog(List<MarkTaskRoughLevel> markTasks, Paper paper) {
        //20191108wangliang加入档位落差值计算
        LOG.info("this:{} levelsLog异步回调进来了", this);
        JSONObject js = new JSONObject();
        js.put("paperId", paper.getId());
        js.put("type", "levelsLog");
        MarkTaskJob markTaskJob = new MarkTaskJob(js.toJSONString(), false, paper.getWorkId());
        levelsLog(markTasks, paper, markTaskJob);
        markTaskJobRepo.save(markTaskJob);
        LOG.info("this:{} levelsLog异步回调结束了", this);
    }

    /**
     * 档位落差值计算日志
     *
     * @param markTasks
     * @param paper
     * @param markTaskJob
     */
    public void levelsLog(List<MarkTaskRoughLevel> markTasks, Paper paper, MarkTaskJob markTaskJob) {
        Map<Long, String> levelsMap = markTasks.stream().collect(Collectors.toMap(MarkTaskRoughLevel::getMarkerId, o -> o.getResult().toUpperCase()));
        arbitrateCallback.judge(levelsMap, paper.getLevel(), new ArbitrateResult() {
            @Override
            public void callback(List<ArbitrateCallback.Distance> list) {
                LOG.info("this:{} callback levelsLog异步回调进来了", this);
                String operResult = "-";
                Date date = new Date();
                for (ArbitrateCallback.Distance d : list) {
                    for (MarkTaskRoughLevel m : markTasks) {
                        if (Objects.equals(String.valueOf(d.getC()), m.getResult()) && d.getMarkId().longValue() == m.getMarkerId().longValue()) {
                            MarkLog markLogPrev = markLogRepo.findTopByCreateUserIdAndOperTypeAndSubjectAndExamNumberOrderByCreateTimeDesc(m.getMarkerId(), MarkLogOperType.LEVEl_DIFFERENCE.getId(), m.getSubject().toString(), paper.getExamNumber());
                            JSONObject jsonObject = new JSONObject();
                            jsonObject.put("task", m.getResult());
                            jsonObject.put("paper", paper.getLevel());
                            if (Objects.isNull(markLogPrev)) {//新建日志
                                //加入档位落差值日志
                                Work work = workRepo.findOne(paper.getWorkId());
                                MarkLog markLog = new MarkLog(m.getMarkerId(), m.getMarkerName(), Role.MARKER, m.getSubject(), paper.getExamNumber(), paper.getStudentName(), MarkLogOperType.LEVEl_DIFFERENCE.getId(), paper.getWorkId(), paper.getId(), MarkStage.LEVEL, operResult, String.valueOf(d.getDistance()), jsonObject.toJSONString(), work.getName(), date);
                                markLogRepo.save(markLog);
                            } else {//更新日志
                                operResult = Optional.ofNullable(markLogPrev.getOperDataAfter()).orElse("-");
                                markLogPrev.setOperDataBefore(operResult);
                                markLogPrev.setOperDataAfter(String.valueOf(d.getDistance()));
                                markLogPrev.setRemark(jsonObject.toJSONString());
                                markLogRepo.save(markLogPrev);
                            }
                            //档位偏差
                            MarkLog markLogDevi = markLogRepo.findTopByCreateUserIdAndOperTypeAndSubjectAndExamNumberOrderByCreateTimeDesc(m.getMarkerId(), MarkLogOperType.LEVEl_DEVIATION.getId(), m.getSubject().toString(), paper.getExamNumber());
                            if (Objects.isNull(markLogDevi)) {//新建日志
                                //加入档位偏差值日志
                                Work work = workRepo.findOne(paper.getWorkId());
                                MarkLog markLog = new MarkLog(m.getMarkerId(), m.getMarkerName(), Role.MARKER, m.getSubject(), paper.getExamNumber(), paper.getStudentName(), MarkLogOperType.LEVEl_DEVIATION.getId(), paper.getWorkId(), paper.getId(), MarkStage.LEVEL, operResult, String.valueOf(d.getDeviation()), jsonObject.toJSONString(), work.getName(), date);
                                markLogRepo.save(markLog);
                            } else {//更新日志
                                operResult = Optional.ofNullable(markLogDevi.getOperDataAfter()).orElse("-");
                                markLogDevi.setOperDataBefore(operResult);
                                markLogDevi.setOperDataAfter(String.valueOf(d.getDeviation()));
                                markLogDevi.setRemark(jsonObject.toJSONString());
                                markLogRepo.save(markLogDevi);
                            }
                        }
                    }
                }
                markTaskJobRepo.updateMarkTaskJobByVersion(markTaskJob.getId(), markTaskJob.getVersion());
                LOG.info("this:{},callback levelsLog异步回调结束了", this);
            }
        });
    }

    /**
     * 直接给试卷分档
     *
     * @param paper
     * @param level
     */
    @Transactional
    public Paper levelMarkPaper(Paper paper, String level, boolean isSample) {
        List<Level> levels = levelRepo.findByWorkId(paper.getWorkId());

        paper.determineRoughLevel(level);
        paper.setMarkByLeader(true);
        paper.setRoughSample(isSample);

        List<MarkTaskRoughLevel> markTasks = markTaskRoughLevelRepo.findByPaperId(paper.getId());
        for (MarkTaskRoughLevel markTask : markTasks) {
            markTask.setRejected(false);
            markTask.setResult(level);
            markTask.setLevel(level);
            markTask.setSample(paper.isSample());
        }
        markTaskRoughLevelRepo.save(markTasks);

        paper.setSortNum(calcSortNum(paper, markTasks, levels));
        paperRepo.save(paper);
        levelsLog(markTasks, paper);
        return paper;
    }

    /**
     * 打回重评
     * 更新试卷状态
     * 更新评卷任务状态为打回
     * 如果评卷任务的档位和建议档位一致，不打回该任务
     *
     * @param paper
     * @param redoLevel 建议重评档位
     * @param ranges
     */
    @Transactional
    public Paper reject(Paper paper, String redoLevel, String ranges) {
        paper.roughReject(redoLevel);
        //科组长打回标记
        paper.setRejectedByLeader(true);
        paper.setMarkByLeader(true);
        //打回后撤销标准卷
        paper.setRoughSample(false);
        List<MarkTaskRoughLevel> markTasks = markTaskRoughLevelRepo.findByPaperId(paper.getId());

        List<Level> levels = levelRepo.findByWorkId(paper.getWorkId());
        Map<String, Integer> levelMap = levels.stream().collect(Collectors.toMap(Level::getCode, Level::getLevelValue));

        List<MarkLog> markLogList = new ArrayList<>();
        Date date = new Date();
        for (MarkTaskRoughLevel markTask : markTasks) {
            String[] range = ranges.split(",");
            if (range.length > 0) {
                //加入手动打回日志
                String operResult = "-";
                for (String r : range) {
                    // 建议的档位与其中评委所评档位相同时就不打回给此评委
                    if (Objects.equals(String.valueOf(markTask.getMarkerId()), r) && !redoLevel.equals(markTask.getResult())) {
                        markTask.setRejected(true);
                        markTask.setOriginLevel(markTask.getResult());
                        markTask.setResult(null);
                        markTask.setLevel(null);
                        markTask.setDeviationDirection(calcDeviationDirection(levelMap, markTask.getOriginLevel(), redoLevel));
                        MarkLog markLogPrev = markLogRepo.findTopByCreateUserIdAndOperTypeAndSubjectAndExamNumberOrderByCreateTimeDesc(markTask.getMarkerId(), MarkLogOperType.HANDLE_LEVEl.getId(), markTask.getSubject().toString(), paper.getExamNumber());
                        if (Objects.nonNull(markLogPrev)) {
                            operResult = Optional.ofNullable(markLogPrev.getOperDataAfter()).orElse("-");
                        }
                        Work work = workRepo.findOne(paper.getWorkId());
                        MarkLog markLog = new MarkLog(markTask.getMarkerId(), markTask.getMarkerName(), Role.MARKER, markTask.getSubject(), paper.getExamNumber(), paper.getStudentName(), MarkLogOperType.HANDLE_LEVEl.getId(), paper.getWorkId(), paper.getId(), MarkStage.LEVEL, operResult, markTask.getOriginLevel(), MarkLogOperType.HANDLE_LEVEl.getName(), work.getName(), date);
                        markLogList.add(markLog);
                    }
                }
            }
        }
        markTaskRoughLevelRepo.save(markTasks);
        paperRepo.save(paper);
        markLogRepo.save(markLogList);
        return paper;
    }

    /**
     * 跳过该任务，最后处理
     *
     * @param markTaskId
     */
    @Transactional
    public MarkTaskRoughLevel skip(Long markTaskId) {
        try {
            MarkTaskRoughLevel m = markTaskRoughLevelRepo.findOne(markTaskId);
            Integer randomSeq = markTaskRoughLevelRepo.findRandomSeqByWorkIdAndSubjectAndMarkerIdAndQuestionId(m.getWorkId(), m.getSubject(), m.getMarkerId(), m.getQuestionId());
            if (randomSeq == null) {
                randomSeq = 0;
            }
            m.setRandomSeq(randomSeq + 1);
            markTaskRoughLevelRepo.save(m);
            return m;
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException("该任务可能被占用，跳过失败");
        }

    }

    /**
     * 计算偏差方向
     *
     * @param levelMap
     * @param result   定档档位
     * @param level    评卷员打的档位
     * @return
     */
    public static String calcDeviationDirection(Map<String, Integer> levelMap, String result, String level) {
        if (Objects.nonNull(result) && Objects.nonNull(level)) {
            Integer resultInt = levelMap.get(result);
            Integer levelInt = levelMap.get(level);
            if (resultInt != null && levelInt != null) {
                return String.valueOf(levelInt - resultInt);
            }
        }
        return null;
    }

    public static Integer calcSortNum(Paper paper, List<MarkTaskRoughLevel> markTaskList, List<Level> levels) {
        //计算levelValue和（试评阶段不用）
        if (paper.getTest() == 0 && Objects.nonNull(paper.getLevel())) {
            if (!levels.isEmpty()) {
                Map<String, Integer> map = levels.stream().collect(Collectors.toMap(Level::getCode, Level::getWeight));
                return markTaskList.stream().mapToInt(m -> map.get(m.getResult()) == null ? 0 : map.get(m.getResult())).sum();
            }
        }
        return 0;
    }

}
